/**
 * Test case for apemandocker.
 * Runs with mocha.
 */
"use strict";

const Apemandocker = require('../lib/apemandocker.js'),
    assert = require('assert');

describe('apemandocker', () => {

    let apemandocker = new Apemandocker({}),
        source = `${__dirname}/../doc/mocks/mock-images/node`,
        image = 'testing-apemandocker-01',
        container = 'testing-apemandocker-01-container-01',
        network = 'test-apemandocker-nw-01';

    before((done) => {
        done();
    });

    after((done) => {
        done();
    });

    it('Build an image.', (done) => {
        apemandocker.buildImage(source, {
            tag: image
        }, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Build list images.', (done) => {
        apemandocker.listImages({}, (err, images) => {
            assert.ifError(err);
            assert.ok(images);
            done();
        });
    });

    it('Check if image exists.', (done) => {
        apemandocker.imageExists(image, (exists) => {
            assert.ok(exists);
            apemandocker.imageExists("__invald_image_name__", (exists) => {
                assert.ok(!exists);
                done();
            })
        });
    });

    it('Create a container.', (done) => {
        apemandocker.createContainer(image, {
            name: container
        }, (err) => {
            assert.ifError(err);
            done();
        });
    });


    it('Check container exists.', (done) => {
        apemandocker.containerExists(container, (exists) => {
            assert.ok(exists);
            apemandocker.containerExists('__invalid_name__', (exists) => {
                assert.ok(!exists);
                done();
            });
        });
    });


    it('Start a container.', (done) => {
        apemandocker.startContainer(container, {}, (err) => {
            assert.ifError(err);
            process.nextTick(()=> {
                done();
            });
        });
    });

    it('Exec in container.', (done) => {
        apemandocker.execInContainer(container, ["ls", "-la"], {
        }, (err) => {
            console.log(err);
            assert.ifError(err);
            process.nextTick(()=> {
                done();
            });
        });
    });

    it('Check the container running.', (done) => {
        apemandocker.isContainerRunning(container, (running) => {
            assert.equal(running, true);
            done();
        });
    });

    it('Log the container.', (done) => {
        apemandocker.logContainer(container, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('List containers.', (done) => {
        apemandocker.listContainers({
            all: true
        }, (err, results) => {
            assert.ifError(err);
            assert.ok(results);
            done();
        });
    });

    it('Terminal with container.', (done) => {
        let terminal = apemandocker.terminal(container);
        setTimeout(()=> {
            terminal.kill(0);
            done();
        }, 1000);
    });

    it('Stop a container.', (done) => {
        apemandocker.stopContainer(container, {}, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Remove a container.', (done) => {
        apemandocker.removeContainer(container, {}, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Remove an image.', (done) => {
        apemandocker.removeImage(image, {
            force: true
        }, (err) => {
            assert.ifError(err);
            done();
        });
    });

    it('Create network.', (done) => {
        apemandocker.createNetwork(network, {}, (err) => {
            assert.ifError(err);
            done();
        })
    });

    it('List network.', (done) => {
        apemandocker.listNetworks({}, (err, networks) => {
            assert.ifError(err);
            assert.ok(networks);
            done();
        })
    });

    it('Has network.', (done) => {
        apemandocker.networkExists(network, (exists) => {
            assert.equal(exists, true);
            apemandocker.networkExists("__invalid_network_name__", (exists) => {
                assert.equal(exists, false);
                done();
            });
        });
    });

    it('Remove network.', (done) => {
        apemandocker.removeNetwork(network, {}, (err) => {
            assert.ifError(err);
            done();
        })
    });

});

